

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <lapi.h>

/* for any error messages returned by LAPI */
char err_msg_buf[LAPI_MAX_ERR_STRING];

/* constant for array lengths */
#define ARRAYLEN  10

/* 
** macro to check return code of function calls. keeps return
** code checking logic from needing to be in main logic 
*/
#define CHECK(func_and_args)                                      \
{                                                                 \
    int rc;                                                       \
    if ((rc = (func_and_args)) != LAPI_SUCCESS) {                 \
        LAPI_Msg_string(rc, err_msg_buf);                         \
        fprintf(stderr,                                           \
                "LAPI returns error message: %s, rc = %d\n",      \
                 err_msg_buf, rc);                                \
        exit(1);                                                  \
    }                                                             \
                                                                  \
}
 


/*
**   Example Program to illustrate the use of LAPI_Qenv and LAPI_Senv
**   routines.
**
**   For a set of n tasks 0,1,...,n-1, where n is an even number,
**   each of the above routines is called, and the resulting data
**   is printed.  
**
**   A LAPI handle is initialized and all variables that can be
**   queried using LAPI_Qenv are shown.  Examples of variables
**   that can be set through LAPI_Senv are also shown.  Note that
**   the printing of all Qenv values gives a list of the default
**   values for all variables accessible through the Qenv interface.
**
*/


int main( int argc, char **argv )
{
    lapi_handle_t handle;        /* LAPI handle */
    lapi_info_t   info;          /* Info to pass to LAPI_Init */
    int           task_id;       /* Our LAPI Task id */
    int           num_tasks;     /* Total number of tasks */
    int           query_val;     /* For query results */
    lapi_statistics_t stats;     /* For stats queries */

    /* Not passing any info to init through this struct. */
    bzero(&info, sizeof(lapi_info_t));

    /* Initialize the LAPI handle */
    CHECK((LAPI_Init(&handle, &info)));

    /* Query LAPI for our task id */
    CHECK((LAPI_Qenv(handle, TASK_ID, &task_id)));

    /* Query LAPI for the total number of tasks in the job. */
    CHECK((LAPI_Qenv(handle, NUM_TASKS, &num_tasks)));

    printf("TASK_ID: %d\n",task_id);
    printf("NUM_TASKS: %d\n", num_tasks);

    CHECK((LAPI_Qenv(handle, MAX_UHDR_SZ, &query_val)));
    printf("Default MAX_UHDR_SZ: %d bytes\n", query_val);

#if !defined(__64BIT__)
    CHECK((LAPI_Qenv(handle, MAX_DATA_SZ, &query_val)));
    printf("Default MAX_DATA_SZ: %d bytes\n", query_val);
#endif /* !defined(__64BIT__) */

    CHECK((LAPI_Qenv(handle, ERROR_CHK, &query_val)));
    printf("Default ERROR_CHK: %d\n",query_val);
    
    CHECK((LAPI_Qenv(handle, TIMEOUT, &query_val)));
    printf("Default TIMEOUT: %d seconds\n", query_val);

    CHECK((LAPI_Qenv(handle, MIN_TIMEOUT, &query_val)));
    printf("Default MIN_TIMEOUT: %d seconds\n", query_val);

    CHECK((LAPI_Qenv(handle, MAX_TIMEOUT, &query_val)));
    printf("Default MAX_TIMEOUT: %d seconds\n", query_val);

    /* set timeout to MAX_TIMEOUT */
    CHECK((LAPI_Senv(handle, TIMEOUT, query_val-1)));
    CHECK((LAPI_Qenv(handle, TIMEOUT, &query_val)));
    printf("After reset, TIMEOUT: %d seconds\n", query_val);

    CHECK((LAPI_Qenv(handle, INTERRUPT_SET, &query_val)));
    printf("Default INTERRUPT_SET: %d\n", query_val);
    /* turn interrupts off */
    CHECK((LAPI_Senv(handle, INTERRUPT_SET, 0)));
    CHECK((LAPI_Qenv(handle, INTERRUPT_SET, &query_val)));
    printf("After reset, INTERRUPT_SET: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, MAX_PORTS, &query_val)));
    printf("Default MAX_PORTS: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, MAX_PKT_SZ, &query_val)));
    printf("Default MAX_PKT_SZ: %d bytes\n", query_val);

    CHECK((LAPI_Qenv(handle, NUM_REX_BUFS, &query_val)));
    printf("Default NUM_REX_BUFS: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, REX_BUF_SZ, &query_val)));
    printf("Default REX_BUF_SZ: %d bytes\n", query_val);

    CHECK((LAPI_Qenv(handle, LOC_ADDRTBL_SZ, &query_val)));
    printf("Default LOC_ADDRTBL_SZ: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, EPOCH_NUM, &query_val)));
    printf("Default EPOCH_NUM: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, USE_THRESH, &query_val)));
    printf("Default USE_THRESH: %d\n", query_val);

    CHECK((LAPI_Senv(handle, USE_THRESH, 1)));
    CHECK((LAPI_Qenv(handle, USE_THRESH, &query_val)));
    printf("After set, USE_THRESH: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, RCV_FIFO_SIZE, &query_val)));
    printf("Default RCV_FIFO_SIZE: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, MAX_ATOM_SIZE, &query_val)));
    printf("Default MAX_ATOM_SIZE: %d bytes\n", query_val);

    CHECK((LAPI_Qenv(handle, BUF_CP_SIZE, &query_val)));
    printf("Default BUF_CP_SIZE: %d bytes\n", query_val);

    CHECK((LAPI_Qenv(handle, MAX_PKTS_OUT, &query_val)));
    printf("Default MAX_PKTS_OUT: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, ACK_THRESHOLD, &query_val)));
    printf("Default ACK_THRESHOLD: %d\n", query_val);
    CHECK((LAPI_Senv(handle, ACK_THRESHOLD, query_val-6)));
    CHECK((LAPI_Qenv(handle, ACK_THRESHOLD, &query_val)));
    printf("After set, ACK_THRESHOLD: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, QUERY_SHM_ENABLED, &query_val)));
    printf("Default QUERY_SHM_ENABLED: %d\n", query_val);

    CHECK((LAPI_Qenv(handle, QUERY_SHM_NUM_TASKS, &query_val)));
    printf("Default QUERY_SHM_NUM_TASKS: %d\n", query_val);
    if ( query_val > 0 ) { /* shm enabled */
        int *shm_task_list;
        int i;
 
        shm_task_list = (int *) (malloc(sizeof(int)*num_tasks));
        CHECK((LAPI_Qenv(handle, QUERY_SHM_TASKS, shm_task_list)));
        for( i = 0; i < 2; i++ ) {
            printf("task[%d] has shm_task_id %d, (num_tasks:%d)\n", i,
                   shm_task_list[i], num_tasks);
        }
        free(shm_task_list);
    }
    /* Do a Gfence to cause some data transfer before printing stats */
    CHECK((LAPI_Gfence(handle)));

    CHECK((LAPI_Qenv(handle, QUERY_STATISTICS, (int *)&stats)));
    printf("**Values passed back from QUERY_STATISTICS**\n");
    /* Total duplicate packets received */
    printf("stats.Tot_dup_pkt_cnt: %llu\n", stats.Tot_dup_pkt_cnt);      
    /* Total packets retransmit */
    printf("stats.Tot_retrans_pkt_cnt: %llu\n", stats.Tot_retrans_pkt_cnt); 
    /* Total ghost packets received  */
    printf("stats.Tot_gho_pkt_cnt: %llu\n", stats.Tot_gho_pkt_cnt);      
    /* Total packets sent  */
    printf("stats.Tot_pkt_sent_cnt: %llu\n", stats.Tot_pkt_sent_cnt);    
    /* Total packets received  */
    printf("stats.Tot_pkt_recv_cnt: %llu\n", stats.Tot_pkt_recv_cnt);    
    /* Total bytes sent  */
    printf("stats.Tot_data_sent: %llu\n", stats.Tot_data_sent);          
    /* Total bytes received  */
    printf("stats.Tot_data_recv: %llu\n", stats.Tot_data_recv);         
    printf("**End of values passed back from QUERY_STATISTICS**\n");

    CHECK((LAPI_Gfence(handle)));

    printf("**About to call query PRINT_STATISTICS**\n");
    CHECK((LAPI_Qenv(handle, PRINT_STATISTICS, &query_val)));
    printf("**done with PRINT_STATISTICS query**\n");

    /* sync before terminating */
    CHECK((LAPI_Gfence(handle)));

    /* terminate */
    CHECK((LAPI_Term(handle)));

    return 0;
}


