/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2008               */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
#pragma comment(copyright,"5765-543 (C) Copyright IBM Corp. 2008 @(#) 1.2 src/ppe/hpct/source/examples/hpm/redBlackSOR.c, ppe.hpct, ppe_rhya 08/12/01 14:19:18")
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifdef HPM
#include "libhpc.h"
#define hpm_errchk if (hpm_error_count) exit(4)
#endif /* HPM */

void   updateOmega(double *omega);

double rho;
double rho2;
double radius;
double pi;
FILE   *outFile;

int readInput( int *n, int *it, double *rhs, int *writeOutput )
{
    FILE *inpFile;
    int  rc;
    char buf[30];
    char srhs[10];

    if ( ( inpFile = fopen("rbInput.dat", "r")) == NULL )
    {
       printf("Input file not provided: Using default values.\n");
       *n = 511;
       *it = 20;
       *rhs = 2.0;
       *writeOutput = 0;
       return(0);
    }
    else
    {
       rc = fscanf( inpFile, "%s %d", buf, n );
       if ( rc != 2 || strcmp(buf, "MatrixSize:") != 0 )
       {
           printf("ERROR --> reading input file: MatrixSize \n");
           return(2);
       }

       rc = fscanf( inpFile, "%s %d %s %lf", buf, it, srhs, rhs );
       if ( rc != 4 ||
            strcmp(buf, "Iterations:") != 0 || strcmp(srhs, "RHS:") != 0)
       {
           printf("ERROR --> reading input file: Iterations and RHS\n");
           return(3);
       }

       rc = fscanf( inpFile, "%s %d ", buf, writeOutput );
       if ( rc != 2 ||
            strcmp(buf, "Output:") != 0)
       {
           printf("ERROR --> reading input file: Output\n");
           return(5);
       }

       if ( *writeOutput != 0 )
       {
          if ( ( outFile = fopen("outputRBSOR.dat", "w")) == NULL )
          {
              printf("ERROR --> Problems with the output file\n");
              return(6);
          }
       }
       return(0);
   }
}

void saveOutput( u, n, iter, norm, oldNorm )
    int    n, iter;
    double norm, oldNorm;
    double *u;
{
    int  i, k, l, ip;

    printf( "After %d iterations, the convergence rate is: %g\n",
            iter-1, fabs(norm - oldNorm) );

    for (i=0; i<n; i++)
    {
        for ( k=0; k<n; k += 8)
        {
            for (l=0; l<8; l++) fprintf(outFile, "%9.4f ", u[n*i+k+l] );
            fprintf(outFile, "\n");
        }
    }
    printf("Results stored in file: outputRBSOR.dat\n");
}

main(argc, argv)
     int argc;
     char *argv[];
{
    int      i, k, rc;

    int      n, it, writeOutput;
    double   *f, *u;
    double   rhs; 

    double norm, oldNorm, nrm;
    double omega = 1.0;

#ifdef HPM
    hpmInit( 0, "RedBlackSOR" );
    hpm_errchk;
    hpmStart( 1, "All" );
    hpm_errchk;
#endif /* HPM */

    rc = readInput( &n, &it, &rhs, &writeOutput );
    if (rc > 0) 
    {
        printf("Exiting due to error: %d\n", rc);
        exit(1);
    }

    pi = 2.0 * acos(0.0);
    rho = cos(pi / (n-1));
    rho2 = rho * rho;
    norm = rhs * (double) (n * n);

#ifdef PRINT
    printf("Matrix size: %d \n", n );
    printf("Number of iterations: %d \n", it );
#endif /* PRINT */

#ifdef HPM
    hpmStart( 5, "Initialization" );
    hpm_errchk;
#endif /* HPM */

    f = (double *) malloc(sizeof(double) * n * n );
    u = (double *) malloc(sizeof(double) * n * n );

    for ( i=0; i<n; i++ ) 
    {
        for ( k=0; k< n; k++ ) 
        {
            u[n * i + k] = 0.0;
            f[n * i + k] = rhs;
        }
    }

#ifdef HPM
    hpmStop( 5 );
#endif /* HPM */

#ifdef HPM
    hpmStart( 17, "Main Iteration" );
    hpm_errchk;
#endif /* HPM */

    for ( i=1; i<=it; i++ ) 
    { 
	relax( n, &omega, f, u, &nrm); 
	updateOmega( &omega);
        oldNorm = norm;
        norm = nrm;
    }

#ifdef HPM
    hpmStop( 17 );
    hpm_errchk;
#endif /* HPM */

    if ( writeOutput != 0 )
    {
#ifdef HPM
       hpmStart( 8, "Save Output" );
       hpm_errchk;
#endif /* HPM */

       saveOutput( u, n, i, norm, oldNorm );

#ifdef HPM
       hpmStop( 8 );
       hpm_errchk;
#endif /* HPM */
    }

#ifdef HPM
    hpmStop( 1 );
    hpm_errchk;
    hpmTerminate( 0 );
    hpm_errchk;
#endif /* HPM */

    return(0);
}
