/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2008               */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
#pragma comment(copyright,"5765-543 (C) Copyright IBM Corp. 2008 @(#) 1.2 src/ppe/hpct/source/examples/plugins/loc_merge.c, ppe.hpct, ppe_rhya 08/12/01 14:19:39")
/****************************************************************************
 *
 *   loc_merge.so does a local merge on each MPI task separately.
 *   It is identical to the mirror.so plug-in 
 *   except for those MPI tasks that change the hardware counter groups 
 *   in the course of the measurement (e.g. by multiplexing).
 *   The different counter data, which are collected for only
 *   part of the measuring interval, are proportionally extended 
 *   to the whole interval and joined into one big group that is 
 *   entering derived metrics computation.
 *
 *               C. Pospiech ** ACTC
 *
 *  $Id: loc_merge.c,v 1.2 2008/03/04 09:30:59 cp Exp $
 *
 ****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "hpc_data.h"
#include "hpc_agg.h"

#define TRUE_ 1
#define FALSE_ 0

int distributor(void) {

  return(0);
}

int aggregator(int num_in, hpm_event_vector in,
               int *num_out, hpm_event_vector *out,
	       int *is_print_task) {

  hpm_event_vector new = *out;
  double **aggr_time = NULL;
  int i, j, k, ct;
  int num_new = 0;
  int rc=0;

  for (i=0; i<num_in; i++) {
    /*
     * If the hpm_data_item contains rusage data, it is simply copied.
     */
    if ( in[i].is_rusage ) {
      if (( aggr_time = (double**)
	    realloc(aggr_time, (num_new+1)*sizeof(double*))) == NULL )
	return errno;
      /*
       * To have the same data structure as for the non-rusage cases
       * in[i].numdata time values are allocated and are populated
       * with the wallclock value for this data item.
       */
      if (( aggr_time[num_new] = (double*)
	    malloc(in[i].num_data*sizeof(double))) == NULL )
	return errno;
      for (j=0; j<in[i].num_data; j++)
	aggr_time[num_new][j] = in[i].times[HPM_TIME_WALLCLOCK];
      /*
       * The corresponding hpm_data_item is allocated and the values
       * are copied with memcpy. This particularly means that only the
       * pointers, but not their values are copied.
       */
      if (( new = (hpm_event_vector)
	    realloc(new, (num_new+1)*sizeof(hpm_data_item))) == NULL )
	return errno;
      memcpy(new+num_new, in+i, sizeof(hpm_data_item));
      num_new++;
    } else {            /* This ends the rusage case */
      /*
       * The hpm_data_items that have the same value of instr_id
       * (even if it is INSTR_ID_NOT_AVAILABE) are merged into one
       * hpm_data_item.
       *
       * The first step is to check whether new already contains
       * a species of this kind. This is done by a linear search.
       */
      j=0;
      while ( (j < num_new) && (new[j].is_rusage
				|| (new[j].instr_id != in[i].instr_id)
				|| (new[j].is_exclusive 
				    != in[i].is_exclusive)) ) j++;
      if ( j == num_new ) { /* means "not found" */
	if (( aggr_time = (double**)
	      realloc(aggr_time, (num_new+1)*sizeof(double*))) == NULL )
	  return errno;
      /*
       * The aggregation times are just copied.
       */
	if (( aggr_time[num_new] = (double*)
	      malloc(in[i].num_data*sizeof(double))) == NULL )
	  return errno;
	for (j=0; j<in[i].num_data; j++)
	  aggr_time[num_new][j] = in[i].times[HPM_TIME_WALLCLOCK];
      /*
       * The corresponding hpm_data_item is allocated and the values
       * are copied with memcpy. This particularly means that only the
       * pointers, but not their values are copied.
       *
       * This is OK except for the struct member data. Since future
       * reallocs are planned for this member, 
       * it should be allocated by malloc.
       */
	if (( new = (hpm_event_vector)
	      realloc(new, (num_new+1)*sizeof(hpm_data_item))) == NULL )
	  return errno;
	memcpy(new+num_new, in+i, sizeof(hpm_data_item));
	if ((new[num_new].data = (hpm_event_info*)
	     malloc(in[i].num_data*sizeof(hpm_event_info))) == NULL )
	  return errno;
	memcpy(new[num_new].data, in[i].data,
	       in[i].num_data*sizeof(hpm_event_info));
	num_new++;
      } else { /* This ends the "not found" case */
	/*
	 * Now in[i] is the old hpmdata_item and new[j] is the new
	 * position, where in[i] has to be merged in.
	 *
	 * Unfortunately this is more complicated than just concatenating
	 * the data members. Some of the data members may occur in serveral
	 * hpm_data_items. So again we have to search for duplicates.
	 */
	for ( ct=0; ct<in[i].num_data; ct++) {
	  k = 0;
	  while ( (k < new[j].num_data) 
		  && ( 0 != strcmp(new[j].data[k].short_name,
				   in[i].data[ct].short_name)) ) k++;
	  if ( k == new[j].num_data ) { /* means again "not found" */
	    /*
	     * This is a new data member and 
	     * has to be appended to the data new[j].data vector.
	     */
	    if ((new[j].data = (hpm_event_info*)
		 realloc(new[j].data,
			 (++new[j].num_data)*sizeof(hpm_event_info))) == NULL )
	      return errno;
	    /* please note that new[j].num_data == k + 1 */
	    memcpy(new[j].data + k, in[i].data + ct, sizeof(hpm_event_info));
	    /*
	     * Also the aggr_time vector needs to be extended accordingly.
	     */
	    if (( aggr_time[j] = (double*)
		  realloc(aggr_time[j],
			  new[j].num_data*sizeof(double))) == NULL )
	      return errno;
	    aggr_time[j][k] = in[i].times[HPM_TIME_WALLCLOCK];
	  } else {  /* this ends the "not found" case */
	    aggr_time[j][k] += in[i].times[HPM_TIME_WALLCLOCK];
	    new[j].data[k].value += in[i].data[ct].value;
	  } /* this end the true merge case. */
	} /* this ends the loop over the in[i].data members. */
	/*
	 * Now new[j].num_data and new[j].data are merged.
	 * new[j].times still need to be updated.
	 *
	 * The HPM_TIME_START and HPM_TIME_STOP wouln't need any merge.
	 * For simplicity they are handled like the other time components.
	 *
	 * All other members of hpm_data_item are left alone.
	 */
	for ( ct=0; ct<HPM_NTIM; ct++)
	  new[j].times[ct] += in[i].times[ct];
      } /* this ends the merge case on the upper level. */
    } /* this ends the non-rusage case */
  } /* and this ends the loop over the members of hpm_vector "in". */

  /*
   * The whole point of maintaining the aggr_time vector is
   * to scale the values to the total time.
   *
   * To avoid NaNQs this is only done if the aggr_time value
   * is nonzero - well, numerically nonzero, to avoid extensive round up
   * errors.
   */
  for ( j=0; j<num_new; j++)
    for (ct=0; ct<new[j].num_data; ct++)
      if ( aggr_time[j][ct] > 0.000001*new[j].times[HPM_TIME_WALLCLOCK] ) {
	new[j].data[ct].value = (long long) 
	  (0.5 + (new[j].times[HPM_TIME_WALLCLOCK]/aggr_time[j][ct] )
	   * new[j].data[ct].value);
      }

  *out = new;
  *num_out = num_new;

  *is_print_task = TRUE_;  /* every task is printing */

  return(0);
}
