/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* Restricted Materials of IBM                                            */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2008               */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* "@(#) 1.1 src/ppe/hpct/source/include/mpt_f.h, ppe.hpct, ppe_rhya, rhya0837c 08/09/16 10:32:36" */
#ifndef _MT_F_H
#define _MT_F_H

#include <stdlib.h>
#include <stdio.h>
/*----------------------------------------------------------*/
/*    define a structure to hold summarized info            */
/*----------------------------------------------------------*/
struct MT_summarystruct {
        int min_rank;
        int max_rank;
        int med_rank;

        void *min_result;
        void *max_result;
        void *med_result;
        void *avg_result;
        void *sum_result;

        void *all_result;
        void *sorted_all_result;
        int *sorted_rank;

};

struct MT_envstruct {

	int mpirank;
	int xCoord;
	int yCoord;
	int zCoord;
	int xSize;
	int ySize;
	int zSize;
	int procid;
	int ntasks;
	double clockHz;
        int nmpi;

};

struct MT_layoutstruct {

	int *xCoord;
	int *yCoord;
	int *zCoord;
	int *procid;

};

struct MT_tracebufferstruct {

	int number_events;
	double total_buffer; /* in terms of MBytes */
	double used_buffer;
	double free_buffer;
};

struct MT_memorystruct {

	unsigned int max_stack_address;
	unsigned int min_stack_address;
	unsigned int max_heap_address;

};

struct MT_callerstruct {

        char *filepath;
        char *filename;
        char *funcname;
        int lineno;
};

/*----------------------------------*/
/* functions called by the profiler */
/*----------------------------------*/
int mt_trace_event(int);
int mt_output_trace(int);
int mt_output_text(void);


/*----------------------------------*/
/* utility function prototype       */
/*----------------------------------*/

long long mt_get_mpi_counts(int);   	/* number of calls for a MPI */
double mt_get_mpi_bytes(int);       	/* size of data tranfer for a MPI */
double mt_get_mpi_time(int);		/* time used for a MPI */
double mt_get_avg_hops(void);           /* average hops each MPI message travels */
double mt_get_time(void);		/* time from the MPI_Init */
double mt_get_elapsed_time(void);       /* time between MPI_Finalize and MPI_Init */

char *mt_get_mpi_name(int);		/* name for a MPI */


int mt_get_tracebufferinfo(struct MT_tracebufferstruct *); /* info for trace buffer */
int mt_get_memoryinfo(struct MT_memorystruct *);	/* info for the stack/heap */
int mt_get_calleraddress(void);	/* info for the caller */
int mt_get_callerinfo(int, struct MT_callerstruct *); /* get caller detailed info */

void mt_get_environment(struct MT_envstruct *);		/* self id info e.g., mpi rank */

void mt_get_layout(struct MT_layoutstruct *);		/* id info for all nodes */
int mt_get_allresults(int , int , struct MT_summarystruct *); /* get global info */

int mt_print_message_size_dist(FILE *);
void mt_print_flat_profile(FILE *);
void mt_print_memory_stats(FILE *);

void mt_trace_start(void);
void mt_trace_stop(void);

/*--------------------------------------------*/
/* fortran-to-c interface with one underscore */
/*--------------------------------------------*/

long long mt_get_mpi_counts_(int);       /* number of calls for a MPI */
double mt_get_mpi_bytes_(int);           /* size of data tranfer for a MPI */
double mt_get_mpi_time_(int);            /* time used for a MPI */
double mt_get_avg_hops_(void);           /* average hops each MPI message travels */
double mt_get_time_(void);               /* time from the MPI_Init */
double mt_get_elapsed_time_(void);       /* time between MPI_Finalize and MPI_Init */

char *mt_get_mpi_name_(int);             /* name for a MPI */


int mt_get_tracebufferinfo_(struct MT_tracebufferstruct *); /* info for trace buffer */
int mt_get_memoryinfo_(struct MT_memorystruct *);        /* info for the stack/heap */
int mt_get_calleraddress_(void);    /* info for the caller */
int mt_get_callerinfo_(int, struct MT_callerstruct *); /* get caller detailed info */

void mt_get_environment_(struct MT_envstruct *);         /* self id info e.g., mpi rank */

void mt_get_layout_(struct MT_layoutstruct *);           /* id info for all nodes */
int mt_get_allresults_(int , int , struct MT_summarystruct *); /* get global info */

int mt_print_message_size_dist_(FILE *);
void mt_print_flat_profile_(FILE *);
void mt_print_memory_stats_(FILE *);

void mt_trace_start_(void);
void mt_trace_stop_(void);


#endif /* define _MT_F_H */
